//! Exports conversations from ChatGPT dumps as Markdown
//! 
//! Example:
//! 
//! ```bash
//! cargo run -- list < conversations.json > to-export.txt
//! nano to-export.txt # Remove any conversations you don't want exported
//! cargo run -- export to-export.txt < conversations.json
//! ```

use std::collections::BTreeMap;

use anyhow::{bail, Result};
use serde::Deserialize;

#[derive(Deserialize)]
struct Conversation {
    id: String,
    title: String,
    mapping: BTreeMap<String, Event>,
}

#[derive(Deserialize)]
struct Event {
    id: String,
    message: Option<Message>,
}

#[derive(Deserialize)]
struct Message {
    author: Author,
    create_time: Option<f64>,
    content: Content,
}

#[derive(Deserialize)]
struct Author {
    role: String,
}

#[derive(Deserialize)]
struct Content {
    content_type: String,
    parts: Option<Vec<String>>,
}

enum Command {
    Export,
    List,
}

fn main() -> Result<()> {
    let mut args = std::env::args();
    // Skip the exe name
    args.next().unwrap();
    
    let mut cmd = None;
    while let Some(arg) = args.next() {
        match arg.as_str() {
            "export" => cmd = Some(Command::Export),
            "list" => cmd = Some(Command::List),
            _ => bail!("Can't understand command line"),
        }
    }
    
    match cmd {
        Some(Command::Export) => unimplemented!(),
        Some(Command::List) => list_conversations(),
        None => bail!("Subcommand is required, e.g. `list` or `export`"),
    }
}

fn list_conversations() -> Result<()> {
    eprintln!("Awaiting conversation.json on stdin...");
    let rdr = std::io::stdin().lock();
    let conversations: Vec<Conversation> = serde_json::from_reader(rdr)?;
    
    for conv in &conversations {
        println!("{} - {}", conv.id, conv.title);
    }
    
    Ok(())
}
